
#ifndef BEST_H_
#define BEST_H_

#include <stdint.h>


#define BEST_FILE_MAILBOX		"/dev/best_mailbox"
#define BEST_FILE_DISP_DMA		"/dev/best_dma_displ"
#define LOCK_FILE				"/var/lock/best_lock"
#define BEST_FILE_CONFIG_INI	"/opt/CAENels/BEST/config.ini"

#define FFT_LEN					(2048)
#define LEN_BBF                 (32) 		// sizeof(best_buffer_line)/sizeof(double)

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

extern char LIB_BEST_VERSION[];


//===========================================================================//
//                                                                           //
//                            Data functions                                 //
//                                                                           //
//===========================================================================//

/** @defgroup dataFunc Data functions
 *  Functions which read measurement and calulcation data from system
 *  @{
 */

///////////////////////////////////////////////////////////////////////////////
/*! \struct best_buffer_line
* best DMA buffer struct
*/
struct __attribute__((packed)) best_buffer_line {  //addresses with double pointer  *(double)
	double bpm0_posX;       // 0
	double bpm0_posY;       // 1
	double bpm0_i0;         // 2
	double bpm0_rsvd;       // 3
	double bpm1_posX;       // 4
	double bpm1_posY;       // 5
	double bpm1_i0;         // 6
	uint32_t sin;           
	uint32_t cos;           // 7
	double dac[4];          // 8
	uint32_t rsvd0[8];      // 8+4=12
	uint32_t rsvd1[4];      // 12+8/2=16
	float setpts[3];        // 16+4/2=18
	uint32_t status;        //
	uint32_t rsvd2[7];      // 18+(3+1)/2=20
	uint32_t timestamp;     //
	double tetramm0[4];     // 20+(7+1)/2=24
	double tetramm1[4];     // 24+4=28
};

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets arrays of "samples" from display DMA
/// \param length number of samples (each sample is 256 bytes big)
/// \return numbers of samples copies (should be equal to length)
int getBuffer(struct best_buffer_line[], int length);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets arrays of postions and intensity from display DMA
/// \param selector Selects BPM (= TetrAMM), should be 0 or 1
/// \param posX Pointer to array where position X will be written
/// \param posY Pointer to array where position Y will be written
/// \param I0 Pointer to array where Intensity will be written
/// \param length Length of the arrays
/// \return numbers of samples copies (should be equal to length)
int getPosArray_sel(
	int selector,
	double* posX,
	double* posY,
	double* I0,
	unsigned int length
);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets arrays of postions and intensity from display DMA
/// \param selector Selects BPM (= TetrAMM), should be 0 or 1
/// \param posX Pointer to array where position X will be written
/// \param posY Pointer to array where position Y will be written
/// \param I0 Pointer to array where Intensity will be written
/// \param length Length of the arrays on which to perform the average
/// \return numbers of samples copies (should be equal to length)
int getPosScalar_sel(
	int selector,
	double* posX,
	double* posY,
	double* I0,
	unsigned int length
);

///////////////////////////////////////////////////////////////////////////////
/// \brief Same as getPosArray_sel, but only for 1st TetrAMM
int getPosArray(double* posX, double* posY, double* I0, unsigned int length);


///////////////////////////////////////////////////////////////////////////////
/// \brief Gets arrays of voltages from display DMA
/// \param length Length of the arrays
/// \return numbers of samples copies (should be equal to length)
int getVoltageArray(double (*voltages)[4], unsigned int length);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets arrays of voltages one of X PreDAC (the PreDAC is only 1 for 
///         for the time beign)
/// \param sel PreDAC selector (0, for the time beign)
/// \param length Length of the arrays
/// \return numbers of samples copies (should be equal to length), or EINVAL
///         if wrong arguments
int getVoltageArray_sel(int sel, double (*voltages)[4], unsigned int length);


///////////////////////////////////////////////////////////////////////////////
/// \brief Gets arrays of currents from display DMA
/// \param length Length of the arrays
/// \return numbers of samples copies (should be equal to length)
int getCurrentArray(double (*currents)[4], unsigned int length);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets arrays of currents one of two TetrAMMs
/// \param sel TetrAMM selector (0 or 1)
/// \param length Length of the arrays
/// \return numbers of samples copies (should be equal to length), or EINVAL
///         if wrong arguments
int getCurrentArray_sel(int sel, double (*currents)[4], unsigned int length);


///////////////////////////////////////////////////////////////////////////////
/// \brief Calcualtes FFT
/// \param selector Selects source for FFT (
///         offset_adc0_ch0 = 24,
///         offset_pos0_posX = 0,
///         offset_pos0_posY = 1,
///         offset_pos0_I0 = 2,
///         offset_pos1_posX = 3,
///         offset_pos1_posY = 4,
///         offset_pos1_I0 = 5,
///         offset_dac0_ch0 = 16,
///         offset_dac0_ch1 = 17,
///         offset_dac0_ch2 = 18,
///         offset_dac0_ch3 = 19
/// \param outM magnitude output (size \ref FFT_LEN / 2)
/// \param outF frequency output (size \ref FFT_LEN / 2)
/// \param removeDC removes DC component
int getFFT(unsigned int selector, double *outM, double *outF, int removeDC);


///////////////////////////////////////////////////////////////////////////////
/// \brief Get position statistics
/// \param selector Selects which BPM
/// \param mean mean value
/// \param std standard deviation value (full bandwidth)
/// \param std_filt standard deviation value (filtered at 100 Hz)
int getPosStats(int selector, 	
                double* meanX,  double* stdX,  double* stdX_filt,
				double* meanY,  double* stdY,  double* stdY_filt,
				double* meanI0, double* stdI0, double* stdI0_filt);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets display frequency
/// \param freq Frequency in hertz
/// \param use_filter enable filter to prevent aliasing
/// \return 0 on success, -1 on fail
/// \note Access level must be user
int setDisplayFreq(int freq, uint8_t use_filter);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets display frequency
/// \return 0 on success, -1 on fail
int getDisplayFreq(int* freq, uint8_t* use_filter);

///////////////////////////////////////////////////////////////////////////////
/*! \struct best_stats
* best statistics data structure
*/
struct __attribute__((__packed__)) best_stats {
	double filt_freq;
	struct __attribute__((__packed__)) {
		double mean;
		double stdev;
		double stdev_lim;
	} stat[6];
};

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets BPM Statistics
/// \param stats in best_stats structure format
/// \return 0 on success, -1 on fail
int getBPMStats(best_stats* stats);


/** @} */ // end of dataFunc


//===========================================================================//
//                                                                           //
//                        Configuration functions                            //
//                                                                           //
//===========================================================================//

/** @defgroup configFunc Configuration functions
 *  Functions which configure system
 *  @{
 */


///////////////////////////////////////////////////////////////////////////////
/// \brief  Gets BPM scaling parameters for position X and Y in [um].
///			Read values from CONFIG_FILE
/// \param selector Selects BPM (= TetrAMM or EnBOX), 
///    				0 for 1st BPM, 1 for 2nd BPM
/// \param scaleX pointer to scaling value for X in [um]
/// \param scaleY pointer to scaling value for Y in [um]
/// \return 0 on success, -1 on fail
int getBPMscaling_sel(
	int selector,
	double* scaleX,
	double* scaleY
);

///////////////////////////////////////////////////////////////////////////////
/// \brief  Gets BPM scaling parameters of 1st BPM for position X and Y in [um].
///			Read values from CONFIG_FILE
/// \param scaleX scaling value for X in [um]
/// \param scaleY scaling value for Y in [um]
/// \return 0 on success, -1 on fail
int getBPMscaling(double* scaleX, double* scaleY);


///////////////////////////////////////////////////////////////////////////////
/// \brief  Gets RoiX, RoiY, RoiI0max, RoiI0min and Roc of 1st BPM.
///			Read values from CONFIG_FILE.
/// \param roiX double parameter in [um]
/// \param roiY double parameter in [um]
/// \param roiI0min double parameter in [A]
/// \param roiI0max double parameter in [A]
/// \param roc double parameter in percentage
/// \note Access level: all levels
int getROCandROI(  	double *roiX,
					double *roiY,
					double *roiI0min,
					double *roiI0max,
					double *roc );

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets enables for different ROC checks of 1st nad 2nd BPMs.
///	       Read values from FPGA.
/// \param rocX	  enable rocX
/// \param rocY	  enable rocY
/// \param rocI0  enable rocI0
/// \param rocX2  enable rocX2
/// \param rocY2  enable rocY2
/// \param rocI02 enable rocI02
/// \return 0 on success, -1 on fail
/// \note Access level: all levels
int getROCenable(bool *rocX,
                 bool *rocY,
                 bool *rocI0,
                 bool *rocX2,
                 bool *rocY2,
                 bool *rocI02);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets enables for different ROC checks of 1st nad 2nd BPMs.
///	       Writes values to FPGA.
/// \param rocX	  enable rocX
/// \param rocY	  enable rocY
/// \param rocI0  enable rocI0
/// \param rocX2  enable rocX2
/// \param rocY2  enable rocY2
/// \param rocI02 enable rocI02
/// \return 0 on success, -1 on fail
/// \note Access level must be admin
int setROCenable(bool rocX,
                 bool rocY,
                 bool rocI0,
                 bool rocX2,
                 bool rocY2,
                 bool rocI02);

//
enum best_pid_select {
	BEST_PID_SELECT_X = 0,
	BEST_PID_SELECT_Y = 1,
	BEST_PID_SELECT_I0 = 2
};

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets output offset in [V] (output=this+PID value, updated even when PID is not running).
///        Write offset to FPGA.
/// \param pid_sel PID select enum
/// \param offset offset value in [V]
/// \return 0 on success, negative on fail
/// \note Write to FPGA
/// \note Access level must be admin
int setOffsetSingle(enum best_pid_select pid_sel, double offset);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets output offset in [V].
///        Read offset from FPGA.
/// \param pid_sel PID select enum
/// \param offset offset value in [V] 
/// \return 0 on success, negative on fail
/// \note Read from FPGA
/// \note Access level: all levels
int getOffsetSingle(enum best_pid_select pid_sel, double* offset);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets output offsets in [V] (output=this+PID value, updated even when
///        PID is not running).
/// \param offsetX in [V] 
/// \param offsetY in [V] 
/// \param offsetI0 in [V] 
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setOffset(double offsetX, double offsetY, double offsetI0);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets number of samples for window averaging.
/// \param sel (0 = X, 1 = Y, 2 = I0)
/// \param nr_samp Number of samples
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setWindAvg(enum best_pid_select pid_sel, uint32_t nr_samp); 

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets number of samples for window averaging.
///        Read number of samples from FPGA.
/// \param sel (0 = X, 1 = Y, 2 = I0)
/// \param nr_samp Number of samples
/// \return 0 on success, negative on fail
int getWindAvg(enum best_pid_select pid_sel, uint32_t* nr_samp);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets PID parameters (legacy).
/// \param pid_sel (0 = X, 1 = Y, 2 = I0)
/// \param Kp See BEST User's Manual for detailed explanation
/// \param Ki See BEST User's Manual for detailed explanation
/// \param Kd See BEST User's Manual for detailed explanation
/// \param emin See BEST User's Manual for detailed explanation
/// \param Imax See BEST User's Manual for detailed explanation
/// \param Omin See BEST User's Manual for detailed explanation
/// \param Omax See BEST User's Manual for detailed explanation
/// \param Ogain See BEST User's Manual for detailed explanation
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setPIDparams( 	char pid_sel,
					double Kp,
                    double Ki,
                    double Kd,
                    double emin,
                    double Imax,
                    double Omin,
                    double Omax,
                    double Ogain);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets single PID parameter of specific PID.
///        Write PID parameter to FPGA.
/// \param pid_sel (0 = X, 1 = Y, 2 = I0)
/// \param pid_par (0=Kp, 1=Ki, 2=Kd, 3=emin, 4=Imax, 5=Omin, 6=Omax, 7=Ogain)
/// \param param_value
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setPIDparamSingle(char pid_sel, int pid_par, double param_value);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets PID parameters.
///        Read PID parameters from FPGA.
/// \param pid_sel (0 = X, 1 = Y, 2 = I0)
/// \param pid_par (0=Kp, 1=Ki, 2=Kd, 3=emin, 4=Imax, 5=Omin, 6=Omax, 7=Ogain)
/// \param param_value
/// \return 0 on success, negative on fail
/// \note Access level: all levels
int getPIDparamSingle(char pid_sel, int pid_par, double* param_value);

////////////////////////////////////////////////////////////////////////////////////////////////////////
/// \brief Sets PID output cross switch. Selects which PID is assigned to which PreDAC output channel.
///        Write PID output cross switch to FPGA.
/// \param posX DAC channel for posX
/// \param posY DAC channel for posY
/// \param I0 DAC channel for I0
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setPIDoutSel(uint8_t posX, uint8_t posY, uint8_t I0);

////////////////////////////////////////////////////////////////////////////////////////////////////////
/// \brief Gets PID output cross switch.
///        Read PID output cross switch from FPGA.
/// \param posX DAC channel for posX
/// \param posY DAC channel for posY
/// \param I0 DAC channel for I0
/// \return 0 on success, negative on fail
/// \note Access level: all levels
int getPIDoutSel(uint8_t *posX, uint8_t *posY, uint8_t *I0);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets input cross switch.
/// \param sel Selects TetrAmm (in case of 2 TetrAmms)
/// \param ch0 
/// \param ch1
/// \param ch2
/// \param ch3
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setCrossBar(uint8_t sel, uint8_t ch0, uint8_t ch1, uint8_t ch2, uint8_t ch3);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets BPM orientation for diffOverSum.
/// \param sel Selects TetrAmm (in case of 2 TetrAmms)
/// \param is90deg 
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setBPMorient(uint8_t sel, uint8_t is90deg);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets value of output mux.
/// \param sel Selects between FPGA (value 1) and software (value 0)
/// \return 0 on success, negative on fail
/// \note Access level must be user or admin
int setOutputMux(uint8_t sel);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets value of output mux.
/// \param sel Selects between FPGA (value 1) and software (value 0)
/// \return 0 on success, negative on fail
/// \note Access level: all levels
int getOutputMux(uint8_t* sel);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets BPM scaling parameters for position X and Y in [um].
///        Write BPM scaling parameters to FPGA.
/// \param scaleX in [um]
/// \param scaleY in [um]
/// \param bpm Selects BPM (= TetrAMM), should be 0 or 1
/// \return 0 on success, -1 on fail
/// \note Access level must be admin
int setBPMscale(double scaleX, double scaleY, int bpm);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets BPM scaling parameters for position X and Y in [um].
///        Read BPM scaling parameters from FPGA.
/// \param selector Selects BPM (= TetrAMM), should be 0 or 1
/// \return 0 on success, -1 on fail
/// \note Access level: all levels
int getBPMscale(int selector, double* scaleX, double* scaleY);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets BPM offset in [um] ONLY FOR TETRAMMs.
///        Write BPM offsets to FPGA.
/// \param offsetX in [um]
/// \param offsetY in [um]
/// \param bpm Selects BPM (= TetrAMM), should be 0 or 1
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setBPMoffset(double offsetX, double offsetY, int bpm);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets BPM offset in [um] ONLY FOR Enboxes.
///        Write BPM offsets to FPGA. (>= 1.8-16)
/// \param offsetX in [um]
/// \param offsetY in [um]
/// \param bpm Selects BPM (= Enbox), should be 0 or 1
/// \param encType, 0: Relative (tonic), 1: Absolute (Resolute)
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setBPMoffsetEnbox(double offsetX, double offsetY, int bpm, int encType);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets BPM device for diffOverSum.
///        Write to FPGA. (>= 1.8-18).
/// \param sel Selects the BPM
/// \param isEnbox
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setBPMdevice(uint8_t sel, uint8_t isEnbox);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets the EnBOX type (relative or absolute) .
///        Write to FPGA. (>= 1.8-18).
/// \param sel Selects the BPM
/// \param encType, 0: Relative (tonic), 1: Absolute (Resolute)
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setEnboxType(uint8_t sel, uint8_t encType);

///////////////////////////////////////////////////////////////////////////////
/// \brief Set diffOverSum calculations (for EnBOX).
///        0:Normal the positions are calculated from enc1 and enc2.
///        1:SumDiff the positions are calculated from enc1+enc2 and enc1-enc2.
///        Write to FPGA. (>= 1.8-18).
/// \param sel Selects the BPM
/// \param encType, 0: Normal (enc1, enc2), 1: SumDiff (enc1+enc2, enc1-enc2)
/// \return 0 on success, negative on fail
/// \note Access level must be admin
int setEnboxSumDiff(uint8_t sel, uint8_t isSumDiff);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets BPM offsets in [um] ONLY FOR TETRAMMs.
///        Read BPM offsets from FPGA.
/// \param offsetX in [um]
/// \param offsetY in [um]
/// \param bpm Selects BPM (= TetrAMM), should be 0 or 1
/// \return 0 on success, negative on fail
int getBPMoffset(double *offsetX, double *offsetY, int bpm);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets BPM offsets in [um] ONLY FOR Enboxes.
///        Read BPM offsets from FPGA.
/// \param offsetX in [um]
/// \param offsetY in [um]
/// \param bpm Selects BPM (= TetrAMM), should be 0 or 1
/// \param encType, 0: Relative (tonic), 1: Absolute (Resolute)
/// \return 0 on success, negative on fail
int getBPMoffsetEnbox(double *offsetX, double *offsetY, int bpm, int encType);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets BPM orientation for diffOverSum.
///        Read BPM orientation from FPGA.
/// \param sel Selects TetrAmm (in case of 2 TetrAmms)
/// \param is90deg (0 = 90deg, 1 = 45deg)
/// \return 0 on success, negative on fail
/// \note Access level: all levels
int getBPMorient(uint8_t sel, uint8_t* is90deg);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets BPM sources.
///        Read BPM sources from FPGA.
/// \param posX BPM (= TetrAMM) for posX
/// \param posY BPM (= TetrAMM) for posY
/// \param I0 	BPM (= TetrAMM) for I0
/// \return ACK
/// \note Access level: all levels
int getBPMselector(int *posX, int *posY, int *I0);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets BPM sources.
///        Write BPM sources to FPGA.
/// \param posX BPM (= TetrAMM) for posX
/// \param posY BPM (= TetrAMM) for posY
/// \param I0 	BPM (= TetrAMM) for I0
/// \return ACK
/// \note Access level: all levels
int setBPMselector(int posX, int posY, int I0);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets input cross switch.
///        Read input cross switch from FPGA.
/// \param sel Selects TetrAmm (in case of 2 TetrAmms)
/// \param ch0 Top / Top-Left
/// \param ch1 Right / Top-Right
/// \param ch2 Bottom / Bottom-Right
/// \param ch3 Left / Bottom-Left
/// \return 0 on success, negative on fail
int getCrossBar(uint8_t sel, uint8_t* ch0, uint8_t* ch1, uint8_t* ch2, uint8_t* ch3);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets ROC min and max values.
///        ROC should be in [um] for positions and in [A] for intensity.
///        Write min and max ROC levels of specific axis to FPGA.
/// \param sel (0 = X, 1 = Y, 2 = I0, 3 = X2, 4 = Y2, 5 = I02)
/// \param min 
/// \param max
/// \return 0 on success, negative on fail
/// \note Access level: access level user or admin
int setROClimits(int sel, double min, double max);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets ROC.
///        ROC values are in [um] for positions and in [A] for intensity.
///        Read min and max ROC levels of specific axis from FPGA.
/// \param sel (0 = X, 1 = Y, 2 = I0, 0 = X2, 1 = Y2, 2 = I02)
/// \param min 
/// \param max
/// \return 0 on success, negative on fail
int getROClimits(int sel, double* min, double* max);

/** @} */ // end of configFunc



//===========================================================================//
//                                                                           //
//                        Access control functions                           //
//                                                                           //
//===========================================================================//

/** @defgroup loginFunc Access control functions
 *  Functions which allow to elevate user access level
 *  @{
 */

///////////////////////////////////////////////////////////////////////////////
/// \brief Acquires lock on /var/lock/best_lock.
/// \param usr Username
/// \param pass Password
/// \return login level (0=cruise, 1=user, 2=admin)
/// \note Access level: all levels
int getLock(const char *usr, const char *pass);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets login level, which can be changed with call on getLock().
/// \return login level (0=cruise, 1=user, 2=admin)
/// \note Access level: all levels
int getLockStatus();

/** @} */ // end of loginFunc


//===========================================================================//
//                                                                           //
//                          PID control functions                            //
//                                                                           //
//===========================================================================//

/** @defgroup PIDfunc PID control functions
 *  Functions which control PID behaviour
 *  @{
 */

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets PID status.
/// \return stoppedByUser = 0, stoppedByROC = 1, paused = 2, running = 3
/// \note Access level: all levels
unsigned int getPIDstatus(void);

///////////////////////////////////////////////////////////////////////////////
/// \brief Enables or disables feedback.
/// \return 0 on success, negative on fail (e.g. authentication)
/// \note Access level must be user or admin
int setFBenable(int enable);

///////////////////////////////////////////////////////////////////////////////
/// \brief Resets internal states of controller.
/// \return 0 on success, negative on fail (e.g. authentication)
/// \note Access level must be user or admin 
int setCtrlReset();

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets PID controller configuration.
///        Write PID controller configuration to FPGA.
/// \param conf PID conf (X = 0, XY = 1, XI0 = 2, Y = 3, YI0 = 4, XYI0 = 5, I0 = 6)
/// \return 0 on success, negative on fail (e.g. authentication)
/// \note Access level must be user or admin
int setPIDconf(char conf);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets PID controller configuration.
///        Read PID configuration from FPGA.
/// \param conf return PID conf
/// \return 0 on success, negative on fail
int getPIDconf(char* conf);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets new setpoint in [um].
///        Write setpoint to FPGA.
///		   Before writing to FPGA the setpoint is divided by scaling parameter of 1st BPM.
///        Scaling parameters are read from CONFIG_FILE.
/// \param sel Selects which PID ( X = 0, Y = 1, I0 = 2 )
/// \param setpt Setpoint in [um]
/// \return 0 on success, negative on fail (e.g. authentication)
/// \note Access level must be user or admin
int setSetpoint(char sel, double setpt);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets setpoint in [um].
///        Read setpoint from FPGA.
///		   After reading from FPGA the setpoint is multiplied by scaling parameter.
///        Scaling parameters are read from CONFIG_FILE.
/// \param sel Selects which PID ( X = 0, Y = 1, I0 = 2 )
/// \param setpt Setpoint in [um]
/// \return 0 on success, negative on fail
/// \note Access level: all levels
int getSetpoint(char sel, double* setpt);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets new setpoint (without using config file).
///        Write setpoint to FPGA.
///		   Before writing to FPGA the setpoint is divided by scaling parameter of 1st BPM.
///        Scaling parameters are read from FPGA.
/// \param sel Selects which PID ( X = 0, Y = 1, I0 = 2 )
/// \param setpt Setpoint in [um]
/// \return 0 on success, negative on fail (e.g. authentication)
/// \note Access level must be user or admin
int setSetpoint2(char sel, double setpt);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets new setpoint (without using config file).
///        Read setpoint from FPGA.
///		   After reading from FPGA the setpoint is multiplied by scaling parameter.
///        Scaling parameters are read from FPGA.
/// \param sel Selects which PID ( X = 0, Y = 1, I0 = 2 )
/// \param setpt Setpoint in [um]
/// \return 0 on success, negative on fail
int getSetpoint2(char sel, double* setpt);

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets IIR controller parameter.
/// \param ctrlr_sel Selects controller ( X = 0, Y = 1, I0 = 2 )
/// \param param_is_a Selects if param written is a (when 1) or b (when 0)
/// \param param_sel Selects parameter (0-9)
/// \param param IIR coefficient
/// \return 0 on success, negative on fail
///
/// This function stores the parameter in the temporary storage.
/// After all parameters have been set use IIRcontrollerCommitParams
/// function to download parameters to controller.
int IIRcontrollerSetParam(char ctrlr_sel, char param_is_a, char param_sel, double param);

///////////////////////////////////////////////////////////////////////////////
/// \brief Commits IIR controller parameters.
/// \param ctrlr_sel Selects controller ( X = 0, Y = 1, I0 = 2 )
/// \return 0 on success, negative on fail
int IIRcontrollerCommitParams(char ctrlr_sel);

///////////////////////////////////////////////////////////////////////////////
/// \brief Reads IIR controller parameter.
/// \param ctrlr_sel Selects controller ( X = 0, Y = 1, I0 = 2 )
/// \param param_is_a Selects if param read is a (when 1) or b (when 0)
/// \param param_sel Selects parameter (0-9)
/// \param param IIR coefficient
/// \return 0 on success, negative on fail
///
/// This function returns the parameter from read-back storage (the parameters
/// which are actually being used by IIR controller).
int IIRcontrollerGetParam(char ctrlr_sel, char param_is_a, char param_sel, double* param);

/** @} */ // end of PIDfunc


//===========================================================================//
//                                                                           //
//                       Function gen control functions                      //
//                                                                           //
//===========================================================================//

/** @defgroup FuncGenfunc Function generator functions
 *  @{
 */


///////////////////////////////////////////////////////////////////////////////
/// \brief Configures function generator.
/// \param freq Frequency (in Hz)
/// \param ampl Amplitude of sine wave
/// \param offset Offset of sine wave
/// \return 0 on success, negative on fail
///
/// configuration is commited when the func gen is enabled. if func gen is
/// already running, just call funcGenEnable function again
int funcGenConfig(float freq, float ampl[4], float offset[4]);

///////////////////////////////////////////////////////////////////////////////
/// \brief Enables or disables function generator.
/// \param enable 0 to disable, 1 to enable
int funcGenEnable(char enable);


/** @} */ // end of FuncGenfunc


//===========================================================================//
//                                                                           //
//                               TetrAMM functions                           //
//                                                                           //
//===========================================================================//

/** @defgroup Tetrammfunc TetrAMM functions
 *  @{
 */

///////////////////////////////////////////////////////////////////////////////
/// \brief Sets front-end range.
/// \param range 0 = RNG0, 1 = RNG1
/// \param selector 0 = TetrAMM on SFP A, 1 = TetrAMM on SFP B
/// \note Access level must be admin
int tetrammSetRange(int range, unsigned char selector);


///////////////////////////////////////////////////////////////////////////////
/// \brief Sets TetrAMM HV voltage.
/// \param voltage voltage in volts
/// \param sel 0 = TetrAMM on SFP A, 1 = TetrAMM on SFP B
/// \note Access level must be admin
int tetrammHVSetVoltage(float voltage, unsigned char sel);


///////////////////////////////////////////////////////////////////////////////
/// \brief Enables or disables TetrAMM HV module.
/// \param enable 1 to enable, 0 to disable
/// \param sel 0 = TetrAMM on SFP A, 1 = TetrAMM on SFP B
/// \note Access level must be admin
int tetrammHVenable(int enable, unsigned char sel);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets number of TetrAMMs.
/// \return TetrAMMs number
unsigned int getTetrammsNumber();

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets TetrAMMs. (>= 1.8-14).
/// \param pos_A on SFP A, 0 = No TetrAMM connected, 1 = TetrAMM connected
/// \param pos_A on SFP B, 0 = No TetrAMM connected, 1 = TetrAMM connected
/// \return 0 on success, negative on fail
int getTetramms(int* pos_A, int* pos_B);

/** @} */ // end of Tetrammfunc


//===========================================================================//
//                                                                           //
//                               EnBOX functions                           //
//                                                                           //
//===========================================================================//

/** @defgroup EnBOXfunc EnBOX functions
 *  @{
 */

///////////////////////////////////////////////////////////////////////////////
// \brief EnBOX Select ENCODER.
/// \param enc_type 0 = Tonic, 1 = Absolute
/// \param selector 0 = EnBOX on SFP A, 1 = EnBOX on SFP B
/// \note Access level must be admin
int enboxEncoderSelect(int enc_type, unsigned char selector);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets number of Enbox. (>= 1.8-15).
/// \return Enboxes number
unsigned int getEnboxesNumber();

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets Enbox. (>= 1.8-15).
/// \param pos_A on SFP A, 0 = No Enbox connected, 1 = Enbox connected
/// \param pos_A on SFP B, 0 = No Enbox connected, 1 = Enbox connected
/// \return 0 on success, negative on fail
int getEnboxes(int* pos_A, int* pos_B);



/** @} */ // end of EnBOXfunc

//===========================================================================//
//                                                                           //
//                                 Misc functions                            //
//                                                                           //
//===========================================================================//

/** @defgroup Miscfunc Misc
 *  @{
 */

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets SFP info.
/// \param selector Selects which SFP port (A=0, B=1, ...)
/// \param id
/// \param timestamp
/// \param fw_ver
/// \param ser_nr
/// \return 0 on success, -1 on fail
/// \note Access level: all levels
int getSFPinfo(int selector,
               uint16_t *id,
               uint32_t *timestamp,
               uint32_t *fw_ver,
               uint32_t *ser_nr);

///////////////////////////////////////////////////////////////////////////////
/// \brief Gets System HW, SW and PID versions.
/// \param hwVer hardware version
/// \param swVer software version (main application)
/// \param ctrlVer controller version (pid version)
/// \return 0 on success, -1 on fail
/// \note Access level: all levels
int getSystemVersion(uint32_t* hwVer, uint32_t* swVer, uint32_t* ctrlVer);


/** @} */ // end of Miscfunc

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* BEST_H_ */
